#include "\lw\include\gpib.h"
#include "\lw\include\formatio.h"
#include "bop488.h"

/*======= KEPCO bop488 ====================================================*/
/* LabWindows 2.0 Instrument Driver                                        */
/* Original Release: 1/92                                                      */
/* By: Peter Blanco , Kepco Inc.                                                                    */
/* Originally written in C                                                 */
/* Modification History: None                                              */
/*=========================================================================*/

static char cmd[50];
static int bd;
static char *hex_str;
static int init;
static double bop488_vmax;
static double bop488_imax;
static int bop488_type;
static int bop488_range;
static int bop488_mod;
static int bop488_err;
static int bop488_addr;

int bop488_invalid_integer_range();
int bop488_invalid_real_range();
int bop488_device_closed();
int bop488_write_data();

int bop488_init (int addr)
{
    if(init != 1){
        bop488_vmax=10;
        bop488_imax=10;
        bop488_type=0;
        hex_str="0123456789ABCDEF";
        init=1;
    }

/*  default configuration 0-10v voltage , 0-1v current */

    if (bop488_invalid_integer_range (addr, 0, 30, -1) != 0)
        return bop488_err;

/*  If device has not been opened (bop488_addr = 0), open it. */

    if (bop488_addr <= 0)    {
        CloseInstrDevs("bop488");
        bop488_addr = OpenDev ("", "bop488");
        if (bop488_addr <= 0)    {
            bop488_err = 220;
            return bop488_err;
        }
    }

/*  Change the primary address of the device    */

    if (ibpad (bop488_addr, addr) < 0)   {
        bop488_err = 233;
        return bop488_err;
    }

    bd = ibfind ("GPIB0");
    if (bd <= 0)    {
        bop488_err = 220;
        return bop488_err;
    }

/* Set Remote Enable    */
    ibsre (bd, 1);
    return bop488_err;
}

/*=========================================================================*/

int bop488_config(int mod,int range,int type,double volt,double curr)
{
    if (bop488_device_closed())
        return bop488_err;

    if (bop488_invalid_integer_range(mod,0,1,-1) )
        return bop488_err;

    if (bop488_invalid_integer_range(range,0,3,-2) )
        return bop488_err;

    if (bop488_invalid_integer_range(type,0,1,-3) )
        return bop488_err;

    if (bop488_invalid_real_range(volt,0.0,10000.0,-4) )
        return bop488_err;

    if (bop488_invalid_real_range(curr,0.0,1000.0,-5) )
        return bop488_err;

    bop488_vmax = volt;
    bop488_imax = curr;
    bop488_type = type;
    bop488_range = range;
    bop488_mod = mod;
    bop488_err = 0;
    return 0;
}

int bop488_set(double volt,double curr)
{
    int d1,d2,d3,range,mod;
    char vout[4],iout[4],dstr[4];

    if (bop488_device_closed())
        return bop488_err;


    if (bop488_invalid_real_range(volt,-bop488_vmax,bop488_vmax,-1) )
        return bop488_err;

    if (bop488_invalid_real_range(curr,-bop488_imax,bop488_imax,-2) )
        return bop488_err;

    mod=bop488_mod;
    range=bop488_range;

    if(mod)
        range+=4;
    if(volt<0)
        switch(range){
            case 0:
            case 2:
                range++;
                break;
            case 1:
            case 3:
                range--;
                break;
        }
    if(curr<0)
        switch(range){
            case 4:
            case 6:
                range++;
                break;
            case 5:
            case 7:
                range--;
                break;
        }
    volt=fabs(volt);
    curr=fabs(curr);
    if(bop488_type){
        if(range<4){
            d1=(volt/bop488_vmax)*999;
            d2=(curr/bop488_imax)*99;
            Fmt(vout,"%s<%i[w3p0]",d1);
            Fmt(iout,"%s<%i[w2p0]",d2);
            Fmt(cmd,"%s<%i%s%s",range,vout,iout);
        }
        else{
            d1=(volt/bop488_vmax)*99;
            d2=(curr/bop488_imax)*999;
            Fmt(iout,"%s<%i[w3p0]",d2);
            Fmt(vout,"%s<%i[w2p0]",d1);
            Fmt(cmd,"%s<%i%s%s",range,iout,vout);
        }
     }

     else{
        if(range<4){
            volt=(volt/bop488_vmax)*4095;
            d1=volt/256;
            volt-=(d1*256);
            d2=volt/16;
            d3=volt-(d2*16);
            vout[0]=hex_str[d1];
            vout[1]=hex_str[d2];
            vout[2]=hex_str[d3];

            curr=(curr/bop488_imax)*255;
            d1=curr/16;
            d2=curr-(d1*16);
            iout[0]=hex_str[d1];
            iout[1]=hex_str[d2];

            vout[3]=0;
            iout[2]=0;

            Fmt(cmd,"%s<%i%s%s",range,vout,iout);
        }
        else{
            curr=(curr/bop488_imax)*4095;
            d1=curr/256;
            curr-=(d1*256);
            d2=curr/16;
            d3=curr-(d2*16);
            iout[0]=hex_str[d1];
            iout[1]=hex_str[d2];
            iout[2]=hex_str[d3];

            volt=(volt/bop488_vmax)*255;
            d1=volt/16;
            d2=volt-(d1*16);
            vout[0]=hex_str[d1];
            vout[1]=hex_str[d2];

            vout[2]=0;
            iout[3]=0;

            Fmt(cmd,"%s<%i%s%s",range,iout,vout);
        }
    }
    return bop488_write_data(cmd,6);
}
/*=========================================================================*/
/* This function removes the instrument from the GPIB device table and     */
/* sets bd = 0.                                                            */
/*=========================================================================*/
int bop488_close ()
{
/*  Check for device closed */

    if (bop488_device_closed())
        return bop488_err;

/*  Close the device    */

    if (CloseDev (bd) < 0)
        bop488_err = 221;
    else
        bop488_err = 0;

    bd = 0;
    return bop488_err;
}

/*= UTILITY ROUTINES ======================================================*/

/*=========================================================================*/
/* This function checks an integer to see if it lies between a minimum     */
/* and maximum value.  If the value is out of range, set the global error  */
/* variable to the value err_code.  The return value is equal 0 for        */
/* success and -1 for error.                                               */
/*=========================================================================*/
int bop488_invalid_integer_range (val, min, max, err_code)
int val;
int min;
int max;
int err_code;
{
  if ((val < min) || (val > max)) {
    bop488_err = err_code;
    return -1;
  }
  return 0;
}

/*=========================================================================*/
/* This function checks a real number to see if it lies between a minimum  */
/* and maximum value.  If the value is out of range, set the global error  */
/* variable to the value err_code.  The return value is equal 0 for        */
/* success and -1 for error.                                               */
/*=========================================================================*/
int bop488_invalid_real_range (val, min, max, err_code)
double val;
double min;
double max;
int err_code;
{
  if ((val < min) || (val > max)) {
    bop488_err = err_code;
    return -1;
  }
  return 0;
}

/*=========================================================================*/
/* This function checks to see if the module has been initialized.  If the */
/* device has not been opened, a -1 is returned, 0 otherwise.              */
/*=========================================================================*/
int bop488_device_closed ()
{
    if (bd <= 0)  {
        bop488_err = 232;
        return -1;
    }
    return 0;
}

/*=========================================================================*/
/* This function writes a buffer of data to the GPIB interface.  The       */
/* return value is equal to the global error variable.                     */
/*=========================================================================*/
int bop488_write_data (buf, cnt)
char *buf;
int cnt;
{
    if (ibwrt(bop488_addr, buf, (long)cnt) <= 0)
        bop488_err = 230;
    else
        bop488_err = 0;

    return bop488_err;
}


